from collections import defaultdict
from pathlib import Path
import os
import yaml

import numpy as np
from pandas import DataFrame
import matplotlib.pyplot as plt

from opa_tools.utils import iterdict, ColorInterpolator

figs = []

COLORS = iterdict(["blue", "red", "green", "orange", "yellow", "purple", "brown"], cycle=False)
MARKERS = iterdict(["s", "o", "^", "v", ">", "<"], cycle=False)

LINES = iterdict(["-", "--", "-.", ":"])
HELPER_FILE = Path(__file__).parent.resolve() / "helperdata.yaml"

DF_COLS = ["method", "ndigests", "init_size", "workers", "weight", "benchmark"]

plt.rcParams["font.family"] = "Arial"


def get_df():
    def read(directory, file):
        with open(directory / f"{file}.yaml", "r", encoding="utf-8") as rf:
            return yaml.safe_load(rf)

    try:
        benchmark_dir = Path(os.environ["BENCHMARK_DIR"]) / "crick"
    except KeyError as exc:
        raise EnvironmentError(
            "Please set environment variable 'BENCHMARK_DIR'."
        ) from exc
    if not benchmark_dir.exists():
        raise FileNotFoundError(
            f"Directory '{benchmark_dir} does not exist."
        )
    data = defaultdict(dict)
    for directory in benchmark_dir.iterdir():
        dname = directory.name
        data[dname]["datetime"] = read(directory, "datetime")
        data[dname]["benchmark"] = read(directory, "benchmark")
        data[dname].update(
            read(directory, "args")
        )
        data[dname].update({
            key: value
            for key, value
            in read(directory, "environ").items()
            if not key.startswith("BASH_")
        })
        data[dname]["argv"] = read(directory, "argv")
    
    # Create dataframe and fill weight with 1 (which was default when producing data)
    df = DataFrame.from_dict(data, orient="index")
    df["weight"] = df["weight"].fillna(1)
    
    # Place DF_COLS first
    columns = DF_COLS + [column for column in df.columns if column not in DF_COLS]
    df = df[columns]

    return df



def figure1(df):
    fig, ax = plt.subplots()
    for method, df_method in df.groupby("method"):
        method_color = COLORS[method]
        workers_values = sorted(set(df_method["workers"]))
        interpolator = ColorInterpolator(0.3, 1.0, len(workers_values))
        for workers, df_workers in df_method.groupby("workers"):
            label = (
                f"{method.upper()} (np={workers})"
                if method in ["mp", "mpi"]
                else method.upper()
            )
            df_sorted = df_workers.sort_values("ndigests")
            xx = df_sorted["ndigests"]
            yy = df_sorted["benchmark"]
            color = interpolator(method_color, workers_values.index(workers))
            ax.plot(xx, yy, marker="o", color=color, label=label)
    ax.set_xscale("log")
    ax.set_yscale("log")
    ax.legend(loc="lower right", ncols=2, fontsize=9)
    ax.set_xlabel("Sequence size")
    ax.set_ylabel("Sequence update time [s]")
    return fig

def figure2(df):
    fig, ax = plt.subplots(figsize=(6, 9))
    for ndigests, df_ndigests in df.groupby("ndigests"):
        color = COLORS[ndigests]
        for method, df_method in df_ndigests.groupby("method"):
            marker = MARKERS[method]
            label = f"{method.upper()} $(N=10^{int(np.log10(ndigests))})$"
            df_sorted = df_method.sort_values("workers")
            xx = df_sorted["workers"]
            yy = df_sorted["benchmark"]
            ax.plot(xx, yy, marker=marker, color=color, label=label)
            if len(xx) == 1:
                ax.axhline(yy.iloc[0], color=color, ls="--", lw=1)

    ax.set_xscale("linear")
    ax.set_yscale("log")
    ax.legend(loc="lower right", ncols=4, fontsize=7)
    ax.set_xlabel("Number of processes (np)")
    ax.set_ylabel("Sequence update time [s]")
    return fig



def main():
    df = get_df()

    df.sort_values("datetime").to_csv("benchmarks.csv")
    df = df[df["init_size"] == 1000]

    df_weight_01 = df[df["weight"] == 1]
    df_weight_24 = df[df["weight"] == 24]

    fig1 = figure1(df_weight_01)
    fig1.suptitle("$w=1$")
    fig1.tight_layout()
    fig1.savefig("Figure1.png", dpi=500)

    fig2 = figure2(df_weight_01)
    fig2.suptitle("$w=1$")
    fig2.tight_layout()
    fig2.savefig("Figure2.png", dpi=500)

    fig3 = figure1(df_weight_24)
    fig3.suptitle("$w=24$")
    fig3.tight_layout()
    fig3.savefig("Figure3.png", dpi=500)

    fig4 = figure2(df_weight_24)
    fig4.suptitle("$w=24$")
    fig4.tight_layout()
    fig4.savefig("Figure4.png", dpi=500)



        
if __name__ == "__main__":
    main()
